package com.muyu.event.service.impl;
import com.alibaba.fastjson2.JSONObject;
import com.muyu.cache.*;
import com.muyu.common.domain.MessageTemplateType;
import com.muyu.common.domain.SysCar;
import com.muyu.common.domain.SysCarFault;
import com.muyu.common.domain.resp.SysCarVo;
import com.muyu.common.domain.resp.WarnRuleResp;
import com.muyu.common.domain.resp.WarnStrategyResp;
import com.muyu.event.consumer.MessageConsumer;
import com.muyu.event.service.IncidentService;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;
/**
 * @author liuxinyue
 * @Package：com.muyu.event.service.impl
 * @name：IncidentServiceImpl
 * @Date：2024/10/9 15:02
 */
@Log4j2
@Service
public class IncidentServiceImpl implements IncidentService {

    private static  int DURATION_SECONDS = 5;
    private static List<JSONObject> receivedStrings = new ArrayList<>();
    private static ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private static int elapsedSeconds = 0;
    private static String file="elapsed";
    private static List<MessageTemplateType> messageTemplateTypes=null;
    private static Long msgTypeId=null;
    //滑窗时间
    private static Long slideTime=null;
    //增长率
    private static Long slideFrequency=null;
    //预警策略
    @Resource
    private WarnStrategyCacheService warnStrategyCacheService;
    //车辆
    @Resource
    private SysCarCacheService sysCarCacheService;
    //预警规则
    @Resource
    private WarnRuleCacheService warnRuleCacheService;
    //报文模版
    @Resource
    private MessageTemplateTypeCacheService messageTemplateTypeCacheService;
    //kafka的主题名称
    private final String topic="kafka-topic";
    @Autowired
    private SysCarFaultCacheService sysCarFaultCacheService;
    @Override
    public void warnEventProcess(JSONObject jsonObject) throws Exception {
        receivedStrings.add(jsonObject);
        //协议解析:每秒穿过来一个JSONObject jsonObject;  添加进receivedStrings
        //根据这个车辆VIN查询出他对应的车辆类型
        String carVin=null;
        //报文模版的ID
        Integer templateId=null;
        //取出这辆车的carVin
        for (JSONObject receivedString : receivedStrings) {
            carVin = (String) receivedString.get("carVin");
        }
        //这辆车的信息
        SysCar carByVin = null;
        List<SysCarVo> carVoList = sysCarCacheService.get(sysCarCacheService.keyPre());
        Map<String, SysCarVo> carMap = carVoList.stream()
                .collect(Collectors.toMap(SysCarVo::getCarVin, Function.identity()));
        //获取到了这个车辆的信息
        carByVin = carMap.get(carVin);
        //获取到这辆车绑定的报文模版
        templateId=carByVin.getTemplateId();
        //这个是这辆车对应的所有策略
        List<WarnStrategyResp> carWithWarnStrategyList=null;
        List<WarnStrategyResp> warnStrategyResps = warnStrategyCacheService.get(warnStrategyCacheService.keyPre());
        for (WarnStrategyResp warnStrategyResp : warnStrategyResps) {
            //策略中有绑定的车辆ID
            if(warnStrategyResp.getCarTypeId()==carByVin.getCarTypeId()){
                carWithWarnStrategyList.add(warnStrategyResp);
            }
        }
        //该车对应的所有预警规则
        List<WarnRuleResp> warnRuleResp=null;
        List<WarnRuleResp> warnRuleResps = warnRuleCacheService.get(warnRuleCacheService.keyPre());
        for (WarnStrategyResp warnStrategyResp : carWithWarnStrategyList) {
            for (WarnRuleResp ruleResp : warnRuleResps) {
                if(warnStrategyResp.getId().equals(ruleResp.getStrategyId())){
                    warnRuleResp.add(ruleResp);
                }
            }
        }
        //报文模版
        messageTemplateTypes = messageTemplateTypeCacheService.get(messageTemplateTypeCacheService.keyPre());
        for (WarnRuleResp ruleResp : warnRuleResp) {
            //每一个规则他绑定了报文模版里面对应的一个配置 比如:电池,或者车速
            msgTypeId = ruleResp.getMsgTypeId();
            //将规则中对应的滑窗时间赋值为DURATION_SECONDS
            DURATION_SECONDS = Math.toIntExact(ruleResp.getSlideTime());
            slideFrequency = ruleResp.getSlideFrequency();
        }
        // 定义一个任务，每秒执行一次
        Runnable task = new Runnable() {
            @Override
            public void run() {
                // 清理超过的数据
                cleanUpOldStrings();
                // 检查超速条件
                checkForSpeeding();
            }
        };
        // 每隔1秒执行一次任务
        scheduler.scheduleAtFixedRate(task, 0, 1, TimeUnit.SECONDS);
    }
    // 清理超过60秒的数据
    private static void cleanUpOldStrings() {
        long currentTime = System.currentTimeMillis();
        receivedStrings.removeIf(jsonObject ->
                currentTime - jsonObject.getLong("time") > TimeUnit.SECONDS.toMillis(DURATION_SECONDS)
        );
    }
    // 检查是否有超速情况
    private static void checkForSpeeding() {
        if (receivedStrings.size() < 2) return; // 如果数据不足，直接返回
        for (int i = 0; i < receivedStrings.size(); i++) {
            JSONObject current = receivedStrings.get(i);
            JSONObject next = receivedStrings.get(i + 1);
            for (MessageTemplateType messageTemplateType : messageTemplateTypes) {
                if(messageTemplateType.getMessageTemplateTypeId().equals(msgTypeId)){
                    Short currentElapsed = current.getShort(messageTemplateType.getMessageField());
                    Short nextElapsed = next.getShort(messageTemplateType.getMessageField());
                    if (nextElapsed > currentElapsed + slideFrequency) {
                        log.info("出错啦,出错啦,您的"+messageTemplateType.getMessageField()+"不正常,请检查!!!");
                    }
                }
            }
        }
    }
    @Override
    public void eventAlarmProcessing(JSONObject jsonObject) throws Exception {
        String carVin = (String) jsonObject.get("carVin");
        //车辆缓存配置
        List<SysCarVo> carVoList = sysCarCacheService.get(sysCarCacheService.keyPre());
        //这辆车的信息
        SysCar carByVin = null;
        Map<String, SysCarVo> carMap = carVoList.stream()
                .collect(Collectors.toMap(SysCarVo::getCarVin, Function.identity()));
        //获取到了这个车辆的信息
        carByVin = carMap.get(carVin);
        Long carTypeId = carByVin.getCarTypeId();
        //车辆绑定的报警规则
        List<SysCarFault> sysCarFaults = sysCarFaultCacheService.get(sysCarFaultCacheService.keyPre());
        //车辆的所有报警规则都重新存储在sysCarFaultList里面
        List<SysCarFault> sysCarFaultList=null;
        SysCar finalCarByVin = carByVin;
        sysCarFaults.forEach(sysCarFault -> {
            if(sysCarFault.getCarTypeId().equals(carTypeId)){
                sysCarFaultList.add(sysCarFault);
            }
        });
        //报文模版

    }
}
