package com.muyu.vehicle.service.impl;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.redis.service.RedisService;
import com.muyu.rabbitmq.consumer.RabbitMQConsumerUtil;
import com.muyu.rabbitmq.producer.RabbitMQProducerUtil;
import com.muyu.vehicle.domain.MqttServerModel;
import com.muyu.vehicle.domain.req.VehicleConnectionReq;
import com.muyu.vehicle.mapper.VehicleConnectMapper;
import com.muyu.vehicle.service.VehicleConnectService;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Log4j2
public class VehicleConnectServiceImpl extends ServiceImpl<VehicleConnectMapper, VehicleConnectionReq> implements VehicleConnectService {
   @Autowired
   private VehicleConnectMapper vehicleConnectMapper;

    @Autowired private RedisService redisService;

    @Autowired
    private RabbitMQProducerUtil rabbitMQProducerUtil;

    @Override
    public Result<MqttServerModel> getConnect(VehicleConnectionReq vehicleConnectionReq) {
        log.info("车辆连接信息:{}", vehicleConnectionReq);
        //生成密码
        vehicleConnectionReq.setPassword(vehicleConnectionReq.getVehicleVin() + vehicleConnectionReq.getTimestamp()
                + vehicleConnectionReq.getNonce());


        Integer i = vehicleConnectMapper.insertVehicleConnection(vehicleConnectionReq);
        if (i > 0) {
            log.info("车辆预上线成功");
        }else {
            log.info("车辆预上线失败");
        }
        rabbitMQProducerUtil.basicSendMessage("SendVin",vehicleConnectionReq.getVehicleVin());
        //从redis获取信息
        // 获取名为 "ipList" 的列表
        List<String> ipList = redisService.getCacheList("FourIpList");
        if (ipList.isEmpty()) {
            return null;
        }

        // 获取当前使用的索引位置
        String indexStr = redisService.getCacheObject("currentIndex");
        int index = indexStr!= null? Integer.parseInt(indexStr) : 0;
        String selectedIp = ipList.get(index);
        // 获取该 IP 的使用次数
        String countStr = redisService.getCacheMapValue("ipCounts", selectedIp);
        log.info("IP:[{}]车辆连接数:[{}]",selectedIp,countStr);
        int count = countStr!= null? Integer.parseInt(countStr) : 0;
        if (count < 6) {
            // 使用次数加一
            count++;
            redisService.setCacheMapValue("ipCounts", selectedIp, String.valueOf(count));
            // 更新索引
            index = (index + 1) % ipList.size();
            redisService.setCacheObject("currentIndex", String.valueOf(index));
            MqttServerModel mqttServerModel = new MqttServerModel("tcp://" + selectedIp.substring(1, selectedIp.length() - 1) + ":1883", "vehicle");
            return Result.success(mqttServerModel);
        } else {
            // 如果使用次数达到 6 次，跳过该 IP 并更新索引
            index = (index + 1) % ipList.size();
            redisService.setCacheObject("currentIndex", String.valueOf(index));
            return getConnect(vehicleConnectionReq);
        }
    }
}
